/* This file is part of nanolisp, a very small Lisp system.

Copyright (C) 2007  Luca Saiu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

%{
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdbool.h>
#include <string.h>
#include <assert.h>
#include "nanolisp.h"

extern int yylineno;
extern char* yytext;
extern int yyleng;
extern FILE *yyin, *yyout;

int yylex(void);
int yyerror(char*);

/* Yes, this is a non-reentrant parser. I know it's ugly. */
extern nanolisp_sexpression_t nanolisp_result_of_parsing;
%}

%union{
  nanolisp_sexpression_t sexpression;
}

%token DOT
%token OPENPAR
%token OPENPAR_ARRAY
%token CLOSEPAR
%token QUOTE
%token QUASIQUOTE
%token UNQUOTE
%token UNQUOTE_SPLICING
%token INTEGER
%token FLOAT
%token SYMBOL
%token TRUE_
%token FALSE_
%token STRING
%type <sexpression> INTEGER
%type <sexpression> SYMBOL
%type <sexpression> FLOAT
%type <sexpression> STRING

%type <sexpression> sexpression
%type <sexpression> rest_of_list
%type <sexpression> sequence_of_sexpressions

%start main

%%

main :
  sequence_of_sexpressions { nanolisp_result_of_parsing = $1; }
/* sexpression { result_of_parsing = $1; } */
;

/* When we read a sequence of s-expression, returning all elements in a
   list: */
sequence_of_sexpressions :
  /* nothing */                        { $$ = nanolisp_make_empty_list(); }
| sexpression sequence_of_sexpressions { $$ = nanolisp_make_cons($1, $2); }
;

sexpression :
  INTEGER { int i; sscanf(yytext, "%i", &i); $$ = nanolisp_make_integer(i); }
/*| FLOAT   { float f; sscanf(yytext, "%f", &f); $$ = nanolisp_make_float(f); }*/
| SYMBOL  { $$ = nanolisp_make_symbol(yytext); }
| TRUE_   { $$ = nanolisp_make_boolean(true); }
| FALSE_  { $$ = nanolisp_make_boolean(false); }
| OPENPAR CLOSEPAR                 { $$ = nanolisp_make_empty_list(); }
| OPENPAR sexpression rest_of_list { $$ = nanolisp_make_cons($2, $3); }
/*
| OPENPAR_ARRAY sequence_of_sexpressions CLOSEPAR
   { $$ = nanolisp_list_to_array($2); }
*/
| QUOTE sexpression
    { $$ = nanolisp_make_cons(nanolisp_make_symbol("quote"),
                              nanolisp_make_cons($2,
                                                 nanolisp_make_empty_list())); }
| QUASIQUOTE sexpression
    { $$ = nanolisp_make_cons(nanolisp_make_symbol("quasiquote"),
                              nanolisp_make_cons($2,
                                                 nanolisp_make_empty_list())); }
| UNQUOTE sexpression
    { $$ = nanolisp_make_cons(nanolisp_make_symbol("unquote"),
                              nanolisp_make_cons($2,
                                                 nanolisp_make_empty_list())); }
| UNQUOTE_SPLICING sexpression
    { $$ = nanolisp_make_cons(nanolisp_make_symbol("unquote-splicing"),
                              nanolisp_make_cons($2,
                              nanolisp_make_empty_list())); }
;

rest_of_list :
  CLOSEPAR                 { $$ = nanolisp_make_empty_list(); }
| DOT sexpression CLOSEPAR { $$ = $2; }
| sexpression rest_of_list { $$ = nanolisp_make_cons($1, $2); }
;

%%
